/***************************************************************************/
/* operandes et ensembles d'opérandes                                      */

#include <stdio.h>
#include <stdlib.h>

#include "operande.h"

// constructor of différent E.A.
static Toperande* ea_new(TinstEAKind kd, int pp)
{
    Toperande* ret = (Toperande*)malloc(sizeof(*ret));
    ret->kind  = kd;
    ret->pp    = pp;
    ret->u.val = 0;

    return ret;
}
static void ea_delete(Toperande* ea)
{
    switch (ea->kind) {
        case EaImmediate:
        case EaDirectMemory:
        case EaInDirectMemory:
            cst_delete(ea->u.val);
            break;
    }
    free(ea);
}

extern Toperande* ea_newImmediat(Tconst* val)
    { Toperande* ret = ea_new(EaImmediate,0);    ret->u.val=val; return ret; }
extern Toperande* ea_newDirectMemory(Tconst* val)
    { Toperande* ret = TODO; return ret; }
extern Toperande* ea_newInDirectMemory(Tconst* val)
    { Toperande* ret = TODO; return ret; }
extern Toperande* ea_newDirectRegister(int reg)
    { Toperande* ret = TODO; return ret; }
extern Toperande* ea_newInDirectRegister(int reg)
    { Toperande* ret = TODOeg; return ret; }
extern Toperande* ea_newInDirectRegisterPreInc(int reg)
    { Toperande* ret = TODO; return ret; }
extern Toperande* ea_newInDirectRegisterPostInc(int reg)
    { Toperande* ret = TODO; return ret; }

// returns the byte size for coding the E.A. ea for an instruction
// working with word of sz bytes
// Caution: sz must be 1, 2 or 4.
extern int ea_length(const Toperande* ea, int sz)
{
    switch (ea->kind) {
        case EaImmediate:
            return ...
        ...
        default:
            fprintf(stderr,"%s: 0x%2x invalid E.A kind in %s.\n",
                    ds_progname(),ea->kind,__func__);
            exit(1);
    }
}

// writes into the memory pointed to by buf the binary code of
// the E.A. ea for an instruction working with word of sz bytes.
// Caution: sz must be 1, 2 or 4.
// Caution: the buf buffer must be large enough.
// returns the number of byte written into the buffer.
extern int ea_code(char* buf, const Toperande* ea, int sz, int last)
{
    int len=0;
    buf[len] = ea->kind+ea->pp + (last<<4);
    fprintf(stderr,"FATAL:%s not yet implemented\n",__func__);
    exit(1);
    return len;
}

// prints on stdout the operande under human format
extern void ea_codeHum(const Toperande* ea, int sz, int last)
{
    int val;
    int len=0;
    switch (ea->kind) {
        case EaImmediate:
          val = cst_eval(ea->u.val);
          switch (sz) {
            case 1: printf(" $0x%02x", val&0xff); break;
            case 2: printf(" $0x%04x", val&0xffff); break;
            case 4: printf(" $0x%08x", val); break;
            default:printf(" $?sz=%d?",sz); break;
          }
          break;
        case EaDirectMemory:
          val = cst_eval(ea->u.val);
          printf(" @%04x", val&0xffff);
          break;
        case EaInDirectMemory:
          val = cst_eval(ea->u.val);
          printf(" [%04x]", val&0xffff);
          break;
        case EaDirectRegister:
          val = ea->u.reg;
          printf(" %%%02d", val&0xf);
          break;
        case EaInDirectRegister:
          val = ea->u.reg;
          printf(" %s[%%%02d]%s", ea->pp==0x40 ? "++" : "", val&0xf, ea->pp==0x20 ? "++" : "");
          break;
        default:
            fprintf(stderr,"%s: 0%2x invalid E.A kind in %s.\n",
                    ds_progname(),ea->kind,__func__);
            exit(1);
    }
}

/***************************************************************************/
/* effective address set                                                   */

extern ToperandeSet* eas_new()
{
    ToperandeSet* ret = (ToperandeSet*)malloc(sizeof(*ret));
    ret->nb=0;
    return ret;
}
extern void          eas_delete(ToperandeSet*eas)
{
    int i;
    for (i=0;i<eas->nb;i+=1)
        ea_delete(eas->t[i]);
    free(eas);
}

// return: 1 for full otherwise 0.
extern int           eas_isFull(const ToperandeSet*eas)
{ return eas->nb == EA_MaxNbOpe; }

extern void          eas_add(ToperandeSet*eas, Toperande* ea)
{
    fprintf(stderr,"FATAL:%s not yet implemented\n",__func__);
    exit(1);
}

extern int eas_length(const ToperandeSet* eas, int sz)
{
    fprintf(stderr,"FATAL:%s not yet implemented\n",__func__);
    exit(1);
}

extern int eas_code(char* buf, const ToperandeSet* eas, int sz)
{
    fprintf(stderr,"FATAL:%s not yet implemented\n",__func__);
    exit(1);
}
